import { visit } from 'unist-util-visit';
export default function makeRemarkCodeBlocks({ tagName }) {
    return function remarkCodeBlocks() {
        return function transformer(tree) {
            const visitor = function (node, index, parent) {
                if (node.lang && parent && index !== null) {
                    const { lang, meta } = node;
                    try {
                        let metaAttributes = {};
                        if (meta) {
                            const metaMatches = Array.from(meta.matchAll(/([^=\s]+)=['"]([^'"\s]+)/g));
                            metaAttributes = metaMatches.reduce((accum, match) => {
                                const [_, key, value] = match;
                                return {
                                    ...accum,
                                    [key]: value,
                                };
                            }, {});
                        }
                        if (!metaAttributes.filename) {
                            node.value = node.value.replace(/^\/\/\s?([^\n]+)\n/, (_, filename) => {
                                metaAttributes.filename = filename;
                                return '';
                            });
                        }
                        const props = {
                            code: node.value,
                            lang,
                            ...metaAttributes,
                        };
                        const attributes = Object.entries(props).map(([name, value]) => ({
                            type: 'mdxJsxAttribute',
                            name,
                            value,
                        }));
                        const codeSnippetWrapper = {
                            type: 'mdxJsxFlowElement',
                            name: tagName,
                            attributes,
                            children: [],
                        };
                        parent.children.splice(index, 1, codeSnippetWrapper);
                    }
                    catch (error) {
                        // eslint-disable-next-line no-console
                        console.error(error);
                    }
                }
            };
            visit(tree, 'code', visitor);
        };
    };
}
